@@ -8,6 +8,7 @@ our @EXPORT = qw(
     skip_all_message
     skip_suite_unless_run_all
     ok_sets_cmp
+    ok_dies
 );
 
 use Time::HiRes 'time';
@@ -44,6 +45,11 @@ sub ok_sets_cmp {
     is_deeply [sort @$a], [sort @$b];
 }
 
+sub ok_dies(&) {
+    eval { shift->() };
+    ok $@;
+}
+
 sub skip_suite_unless_run_all {
     unless ($ENV{NET_FLUIDDB_RUN_FULL_SUITE}) {
        plan skip_all => "set NET_FLUIDDB_RUN_FULL_SUITE to run these";
